import re
from pathlib import Path

from packaging.version import parse as _parse
from sphinx import addnodes

import ablog

ablog_builder = "dirhtml"
ablog_website = "_website"
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.doctest",
    "sphinx.ext.intersphinx",
    "sphinx.ext.todo",
    "sphinx.ext.ifconfig",
    "sphinx.ext.extlinks",
    "sphinx_automodapi.automodapi",
    "ablog",
    "alabaster",
    "nbsphinx",
    "myst_parser",
]

version = str(_parse(ablog.__version__))
project = "ABlog"
copyright = "2014-2022, ABlog Team"
master_doc = "index"
source_suffix = {
    ".rst": "restructuredtext",
    ".md": "markdown",
}
exclude_patterns = ["_build", "docs/manual/.ipynb_checkpoints"]
html_title = "ABlog"
html_use_index = True
html_domain_indices = False
html_show_sourcelink = True
html_favicon = "_static/ablog.ico"
blog_title = "ABlog"
blog_baseurl = "https://ablog.readthedocs.io/"
blog_locations = {
    "Pittsburgh": ("Pittsburgh, PA", "https://en.wikipedia.org/wiki/Pittsburgh"),
    "San Fran": ("San Francisco, CA", "https://en.wikipedia.org/wiki/San_Francisco"),
    "Denizli": ("Denizli, Turkey", "https://en.wikipedia.org/wiki/Denizli"),
}
blog_languages = {
    "en": ("English", None),
    "nl": ("Nederlands", None),
    "zh_CN": ("Chinese", None),
}
blog_default_language = "en"
language = "en"
blog_authors = {
    "Ahmet": ("Ahmet Bakan", "https://ahmetbakan.com"),
    "Luc": ("Luc Saffre", "https://saffre-rumma.net/luc/"),
    "Mehmet": ("Mehmet Gerçeker", "https://github.com/mehmetg"),
    "Libor": ("Libor Jelínek", "https://liborjelinek.github.io/"),
}
blog_feed_archives = True
blog_feed_fulltext = True
blog_feed_templates = {
    "atom": {
        "content": "{{ title }}{% for tag in post.tags %} #{{ tag.name|trim()|replace(' ', '') }}{% endfor %}",
    },
    "social": {
        "content": "{{ title }}{% for tag in post.tags %} #{{ tag.name|trim()|replace(' ', '') }}{% endfor %}",
    },
}
disqus_shortname = "https-ablog-readthedocs-io"
disqus_pages = True
fontawesome_link_cdn = "https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.1.1/css/all.min.css"
html_theme = "alabaster"
html_sidebars = {
    "**": [
        "about.html",  # Comes from alabaster
        "searchfield.html",  # Comes from alabaster
        "ablog/postcard.html",
        "ablog/recentposts.html",
        "ablog/tagcloud.html",
        "ablog/categories.html",
        "ablog/archives.html",
        "ablog/authors.html",
        "ablog/languages.html",
        "ablog/locations.html",
    ]
}
html_theme_options = {
    "travis_button": False,
    "github_user": "sunpy",
    "github_repo": "ablog",
    "description": "ABlog for blogging with Sphinx",
    "logo": "ablog.png",
}
intersphinx_mapping = {
    "python": ("https://docs.python.org/", None),
    "sphinx": ("https://www.sphinx-doc.org/en/master/", None),
}
extlinks = {
    "wiki": ("https://en.wikipedia.org/wiki/%s", "%s"),
    "issue": ("https://github.com/sunpy/ablog/issues/%s", "issue %s"),
    "pull": ("https://github.com/sunpy/ablog/pull/%s", "pull request %s"),
}
rst_epilog = """
.. _Sphinx: http://sphinx-doc.org/
.. _Python: https://python.org
.. _Disqus: https://disqus.com/
.. _GitHub: https://github.com/sunpy/ablog
.. _PyPI: https://pypi.python.org/pypi/ablog
.. _Read The Docs: https://readthedocs.org/
.. _Alabaster: https://github.com/bitprophet/alabaster
"""
locale_dirs = [str(Path(ablog.__file__).parent / Path("locales"))]
nitpicky = True
nitpick_ignore = []
for line in open("nitpick-exceptions"):
    if line.strip() == "" or line.startswith("#"):
        continue
    dtype, target = line.split(None, 1)
    target = target.strip()
    nitpick_ignore.append((dtype, target))


def parse_event(env, sig, signode):
    event_sig_re = re.compile(r"([a-zA-Z-]+)\s*\((.*)\)")
    m = event_sig_re.match(sig)
    if not m:
        signode += addnodes.desc_name(sig, sig)
        return sig
    name, args = m.groups()
    signode += addnodes.desc_name(name, name)
    plist = addnodes.desc_parameterlist()
    for arg in args.split(","):
        arg = arg.strip()
        plist += addnodes.desc_parameter(arg, arg)
    signode += plist
    return name


def setup(app):
    from sphinx.ext.autodoc import cut_lines
    from sphinx.util.docfields import GroupedField

    app.connect("autodoc-process-docstring", cut_lines(4, what=["module"]))
    app.add_object_type(
        "confval",
        "confval",
        objname="configuration value",
        indextemplate="pair: %s; configuration value",
    )
    fdesc = GroupedField("parameter", label="Parameters", names=["param"], can_collapse=True)
    app.add_object_type("event", "event", "pair: %s; event", parse_event, doc_field_types=[fdesc])



try:
    from ._version import version
except Exception:
    import warnings

    warnings.warn(f"could not determine {__name__.split('.')[0]} package version; this indicates a broken installation")
    del warnings

    version = "0.0.0"

from packaging.version import parse as _parse

_version = _parse(version)
major, minor, bugfix = [*_version.release, 0][:3]
release = not _version.is_devrelease

__all__ = ["version", "major", "minor", "bugfix", "release"]



"""
ABlog for Sphinx.
"""

import os
from glob import glob
from pathlib import PurePath

from sphinx.builders.html import StandaloneHTMLBuilder
from sphinx.errors import ThemeError
from sphinx.jinja2glue import BuiltinTemplateLoader, SphinxFileSystemLoader
from sphinx.locale import get_translation

from .blog import CONFIG, Blog
from .post import (
    CheckFrontMatter,
    PostDirective,
    PostListDirective,
    UpdateDirective,
    UpdateNode,
    generate_archive_pages,
    generate_atom_feeds,
    missing_reference,
    process_postlist,
    process_posts,
    purge_posts,
)
from .version import version as __version__

__all__ = ["setup", "__version__"]

PKGDIR = os.path.abspath(os.path.dirname(__file__))
# Name used for the *.pot, *.po and *.mo files
MESSAGE_CATALOG_NAME = "sphinx"
_ = get_translation(MESSAGE_CATALOG_NAME)  # NOQA


def get_html_templates_path():
    """
    Return path to ABlog templates folder.
    """
    pkgdir = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(pkgdir, "templates")


def anchor(post):
    """
    Return anchor string for posts that are page sections.
    """
    if post.section:
        return "#" + post.section
    else:
        return ""


def builder_support(builder):
    """
    Return True when builder is supported.

    Supported builders output in html format, but exclude
    `PickleHTMLBuilder` and `JSONHTMLBuilder`, which run into issues
    when serializing blog objects.
    """
    if hasattr(builder, "builder"):
        builder = builder.builder
    not_supported = {"json", "pickle"}
    return builder.format == "html" and builder.name not in not_supported


def html_page_context(app, pagename, templatename, context, doctree):
    if builder_support(app):
        context["ablog"] = blog = Blog(app)
        context["anchor"] = anchor
        if pagename in blog and blog[pagename].canonical_link:
            context["pageurl"] = blog[pagename].canonical_link
        # following is already available for archive pages
        if blog.blog_baseurl and "feed_path" not in context:
            context["feed_path"] = blog.blog_path
            context["feed_title"] = blog.blog_title


def config_inited(app, config):
    # Automatically identify any blog posts if a pattern is specified in the config
    if isinstance(config.blog_post_pattern, str):
        config.blog_post_pattern = [config.blog_post_pattern]
    matched_patterns = []
    for pattern in config.blog_post_pattern:
        pattern = os.path.join(app.srcdir, pattern)
        # make sure that blog post paths have forward slashes even on windows
        matched_patterns.extend(
            PurePath(ii).relative_to(app.srcdir).with_suffix("").as_posix() for ii in glob(pattern, recursive=True)
        )
    app.config.matched_blog_posts = matched_patterns

    # Add ablog stylesheets to static_path.
    static_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "stylesheets"))
    app.config.html_static_path.append(static_path)


def builder_inited(app):
    if not isinstance(app.builder, StandaloneHTMLBuilder) or app.config.skip_injecting_base_ablog_templates:
        return
    if not isinstance(app.builder.templates, BuiltinTemplateLoader):
        raise Exception(
            "Ablog does not know how to inject templates into with custom "
            "template bridges. You can use `ablog.get_html_templates_path()` to "
            "get the path to add in your custom template bridge and set "
            "`skip_injecting_base_ablog_templates = False` in your "
            "`conf.py` file."
        )
    if get_html_templates_path() in app.config.templates_path:
        raise Exception(
            "Found the path from `ablog.get_html_templates_path()` in the "
            "`templates_path` variable from `conf.py`. Doing so interferes "
            "with Ablog's ability to stay compatible with Sphinx themes that "
            "support it out of the box. Please remove `get_html_templates_path` "
            "from `templates_path` in your `conf.py` to resolve this."
        )
    theme = app.builder.theme
    loaders = app.builder.templates.loaders
    templatepathlen = app.builder.templates.templatepathlen
    try:
        # Modern Sphinx now errors instead of returning the default if there is not a value
        # in any of the config files.
        after_theme = theme.get_config("options", "ablog_inject_templates_after_theme", False)
    except ThemeError:
        after_theme = False
    if after_theme:
        # Inject *after* the user templates and the theme templates,
        # allowing themes to override the templates provided by this
        # extension while those templates still serve as a fallback.
        loaders.append(SphinxFileSystemLoader(get_html_templates_path()))
    else:
        # Inject *after* the user templates and *before* the theme
        # templates. This enables ablog to provide support for themes
        # that don't support it out-of-the-box, like alabaster.
        loaders.insert(templatepathlen, SphinxFileSystemLoader(get_html_templates_path()))


def setup(app):
    """
    Setup ABlog extension.
    """
    app.require_sphinx("6.2")
    for args in CONFIG:
        app.add_config_value(*args[:3])
    app.add_directive("post", PostDirective)
    app.add_directive("postlist", PostListDirective)
    app.connect("config-inited", config_inited)
    app.connect("builder-inited", builder_inited)
    app.connect("doctree-read", process_posts)
    app.connect("env-purge-doc", purge_posts)
    app.connect("doctree-resolved", process_postlist)
    app.connect("missing-reference", missing_reference)
    app.connect("html-collect-pages", generate_archive_pages)
    app.connect("html-collect-pages", generate_atom_feeds)
    app.connect("html-page-context", html_page_context)
    app.add_transform(CheckFrontMatter)
    app.add_directive("update", UpdateDirective)
    app.add_node(
        UpdateNode,
        html=(lambda s, n: s.visit_admonition(n), lambda s, n: s.depart_admonition(n)),
        latex=(lambda s, n: s.visit_admonition(n), lambda s, n: s.depart_admonition(n)),
    )
    pkgdir = os.path.abspath(os.path.dirname(__file__))
    locale_dir = os.path.join(pkgdir, "locales")
    app.add_message_catalog(MESSAGE_CATALOG_NAME, locale_dir)
    return {
        "version": __version__,
        "parallel_read_safe": False,
        "parallel_write_safe": True,
    }



from pathlib import Path
import pytest
from sphinx.application import Sphinx
from sphinx.errors import ThemeError


@pytest.mark.sphinx("html", testroot="templates", confoverrides={"skip_injecting_base_ablog_templates": True})
def test_skip_ablog_templates_but_missing_templates(app: Sphinx):
    """
    Completely override the templates used by ablog, but not provide them.
    """
    with pytest.raises(
        ThemeError,
        match=r"An error happened in rendering the page blog/author\.\nReason: TemplateNotFound\(\"'ablog/catalog.html' not found in *.",
    ):
        app.build()


@pytest.mark.sphinx(
    "html",
    testroot="templates",
    confoverrides={
        "templates_path": ["_templates"],
        "skip_injecting_base_ablog_templates": False,  # default value
        "html_sidebars": {
            "**": [
                # overridden by user
                "ablog/postcard.html",
                # fallback to builtin
                "ablog/authors.html",
            ]
        },
    },
)
def test_override_template_but_fallback_missing(app: Sphinx, rootdir: Path):
    """
    Partially override the only some Ablog templates, but use the Ablog ones
    for missing as fallback.
    """
    app.build()

    # is the customized template it in the output?
    customized = (rootdir / "test-templates" / "_templates" / "ablog" / "postcard.html").read_text()
    source = (app.outdir / "post.html").read_text()
    assert customized in source

    # is builtin template in the output?
    builtin = '<div class="ablog-sidebar-item ablog__authors">'
    assert builtin in source


@pytest.mark.sphinx(
    "html",
    testroot="templates",
    confoverrides={
        "html_sidebars": {
            "**": [
                # overridden by theme
                "ablog/postcard.html",
                # fallback to builtin
                "ablog/authors.html",
            ]
        },
        "html_theme_path": "_themes",
        "html_theme": "test_theme",
    },
)
def test_themes_templates_come_first(app: Sphinx, rootdir: Path):
    """
    Ensures that if theme supplies own Ablog template, it is used over the
    builtin one, but fallback to builtin for missing ones.
    """
    app.build()

    # is the customized template it in the output?
    customized = (rootdir / "test-templates" / "_themes" / "test_theme" / "ablog" / "postcard.html").read_text()
    source = (app.outdir / "post.html").read_text()
    assert customized in source

    # is builtin template in the output?
    builtin = '<div class="ablog-sidebar-item ablog__authors">'
    assert builtin in source



import pytest


def read_text(path):
    """
    Support function to give backward compatibility with older sphinx (v2).
    """
    if hasattr(path, "read_text"):
        return path.read_text()
    return path.text()


@pytest.mark.sphinx("html", testroot="canonical")  # using roots/test-canonical
def test_canonical(app, status, warning):
    app.build()

    assert app.statuscode == 0
    assert (app.outdir / "post.html").exists()
    assert (app.outdir / "canonical.html").exists()

    html = read_text(app.outdir / "post.html")
    assert '<link rel="canonical" href="https://blog.example.com/post.html" />' in html

    html = read_text(app.outdir / "canonical.html")
    assert '<link rel="canonical" href="https://canonical.example.org/foo.html" />' in html
